//
//  LC_AppDelegate.h
//  Location Checker
//
//  Created by  Sven on 01.08.06.
//  Copyright earthlingsoft 2006 . All rights reserved.
//

#import "LC_AppDelegate.h"

@implementation LC_AppDelegate


/**
    Returns the support folder for the application, used to store the Core Data
    store file.  This code uses a folder named "Location_Checker" for
    the content, either in the NSApplicationSupportDirectory location or (if the
    former cannot be found), the system's temporary directory.
 */

- (NSString *)applicationSupportFolder {

    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
    return [basePath stringByAppendingPathComponent:@"Location_Checker"];
}


/**
    Creates, retains, and returns the managed object model for the application 
    by merging all of the models found in the application bundle and all of the 
    framework bundles.
 */
 
- (NSManagedObjectModel *)managedObjectModel {

    if (managedObjectModel != nil) {
        return managedObjectModel;
    }
	
    NSMutableSet *allBundles = [[NSMutableSet alloc] init];
    [allBundles addObject: [NSBundle mainBundle]];
    [allBundles addObjectsFromArray: [NSBundle allFrameworks]];
    
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles: [allBundles allObjects]] retain];
    [allBundles release];
    
    return managedObjectModel;
}


/**
    Returns the persistent store coordinator for the application.  This 
    implementation will create and return a coordinator, having added the 
    store for the application to it.  (The folder for the store is created, 
    if necessary.)
 */

- (NSPersistentStoreCoordinator *) persistentStoreCoordinator {

    if (persistentStoreCoordinator != nil) {
        return persistentStoreCoordinator;
    }

    NSFileManager *fileManager;
    NSString *applicationSupportFolder = nil;
    NSURL *url;
    NSError *error;
    
    fileManager = [NSFileManager defaultManager];
    applicationSupportFolder = [self applicationSupportFolder];
    if ( ![fileManager fileExistsAtPath:applicationSupportFolder isDirectory:NULL] ) {
        [fileManager createDirectoryAtPath:applicationSupportFolder attributes:nil];
    }
    
    url = [NSURL fileURLWithPath: [applicationSupportFolder stringByAppendingPathComponent: @"Location_Checker.xml"]];
    persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: [self managedObjectModel]];
    if (![persistentStoreCoordinator addPersistentStoreWithType:NSXMLStoreType configuration:nil URL:url options:nil error:&error]){
        [[NSApplication sharedApplication] presentError:error];
    }    

    return persistentStoreCoordinator;
}


/**
    Returns the managed object context for the application (which is already
    bound to the persistent store coordinator for the application.) 
 */
 
- (NSManagedObjectContext *) managedObjectContext {

    if (managedObjectContext != nil) {
        return managedObjectContext;
    }

    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    }
    
    return managedObjectContext;
}


/**
    Returns the NSUndoManager for the application.  In this case, the manager
    returned is that of the managed object context for the application.
 */
 
- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window {
    return [[self managedObjectContext] undoManager];
}


/**
    Performs the save action for the application, which is to send the save:
    message to the application's managed object context.  Any encountered errors
    are presented to the user.
 */
 
- (IBAction) saveAction:(id)sender {

    NSError *error = nil;
    if (![[self managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
}


/**
    Implementation of the applicationShouldTerminate: method, used here to
    handle the saving of changes in the application managed object context
    before the application terminates.
 */
 
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {

    NSError *error;
    int reply = NSTerminateNow;
    
    if (managedObjectContext != nil) {
        if ([managedObjectContext commitEditing]) {
            if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
				
                // This error handling simply presents error information in a panel with an 
                // "Ok" button, which does not include any attempt at error recovery (meaning, 
                // attempting to fix the error.)  As a result, this implementation will 
                // present the information to the user and then follow up with a panel asking 
                // if the user wishes to "Quit Anyway", without saving the changes.

                // Typically, this process should be altered to include application-specific 
                // recovery steps.  

                BOOL errorResult = [[NSApplication sharedApplication] presentError:error];
				
                if (errorResult == YES) {
                    reply = NSTerminateCancel;
                } 

                else {
					
                    int alertReturn = NSRunAlertPanel(nil, @"Could not save changes while quitting. Quit anyway?" , @"Quit anyway", @"Cancel", nil);
                    if (alertReturn == NSAlertAlternateReturn) {
                        reply = NSTerminateCancel;	
                    }
                }
            }
        } 
        
        else {
            reply = NSTerminateCancel;
        }
    }
    
    return reply;
}


/**
    Implementation of dealloc, to release the retained variables.
 */
 
- (void) dealloc {

    [managedObjectContext release], managedObjectContext = nil;
    [persistentStoreCoordinator release], persistentStoreCoordinator = nil;
    [managedObjectModel release], managedObjectModel = nil;
    [super dealloc];
}



/*
	THE WORKHORSE
*/
- (void) switch:(id) sender {
	// NSString * network = [networkField stringValue];
	NSString * printer = [printerField stringValue];
	NSString * location = [locationField stringValue];
	
	
	//NSLog(@"network:%@, printer:%@, location:%@", network,printer, location);
	
	// switch location
	[self setLocation:location];
	
	// switch default printer
	[self setPrinter:printer];
}


/*
	check whether we want to autoswitch
*/
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"autoswitch"]) {
		// we want to switch automagically
		// find out the current network name first
		
		// ioreg -l -n AirPortDriver | grep APCurrentSSID | sed 's/^.*= "\(.*\)".*$/\1/; s/ /_/g'		
		//CFMutableDictionaryRef  serviceMatch = IOServiceNameMatching( [@"AirPort*" cString]); 
		//io_service_t myService = 0;
		//myService = IOServiceGetMatchingService( kIOMasterPortDefault,  serviceMatch);  
		//IOObjectRelease(myService);
		NSString * airportcheckerPath = [[NSBundle mainBundle] pathForResource:@"airportname" ofType:@"sh"];
		NSTask *airportTask = [[[NSTask alloc] init] autorelease];
		[airportTask setLaunchPath: airportcheckerPath];
		NSPipe *pipe= [NSPipe pipe];
		[airportTask setStandardOutput: pipe];
		NSFileHandle *file = [pipe fileHandleForReading];
		
		[airportTask launch];
		
		NSData *data = [file readDataToEndOfFile];
		NSString * networkName = [[[NSString alloc] initWithData: data encoding: NSUTF8StringEncoding] autorelease];
		networkName = [networkName substringToIndex:[networkName length] -1];
		// NSLog (@"got\n%@", string);
		
		//
		//
		// then find the first entry for that network name
		NSEntityDescription *entityDescription = [NSEntityDescription entityForName:@"Setup" inManagedObjectContext:managedObjectContext];
		NSFetchRequest *request = [[[NSFetchRequest alloc] init] autorelease];
		[request setEntity:entityDescription];
		
		NSPredicate *predicate = [NSPredicate predicateWithFormat:@"(network LIKE %@)", networkName];
		[request setPredicate:predicate];
		
		NSError *error = nil;
		NSArray *array = [managedObjectContext executeFetchRequest:request error:&error];
		if ((array == nil) && [array count])
		{
			// Deal with error...
			NSBeep();
			NSLog(@"Network '%@' not found in list", networkName);
		}
		else {
			NSManagedObject * result = [array objectAtIndex:0];	
			NSString * myPrinter = [result valueForKey:@"printer"];
			// NSLog (myPrinter);
			if (myPrinter) {[self setPrinter:myPrinter];}
			
			NSString * myLocation = [result valueForKey:@"location"];
			if(myLocation) {[self setLocation:myLocation];}
			
			// The NSEvent we get from [NSApp currentEvent] doesn't have the modifier informatiion populated corretly at this stage, so we need to use the Carbon function GetCurrentKeyModifiers()
			if  ([[NSUserDefaults standardUserDefaults] boolForKey:@"autoquit"] && 
					!(     (GetCurrentKeyModifiers() & optionKey) != 0
						|| (GetCurrentKeyModifiers() & cmdKey) != 0 
						|| (GetCurrentKeyModifiers() & shiftKey) != 0
						|| (GetCurrentKeyModifiers() & controlKey) != 0
					 )
				 ) {
				[NSApp terminate:self];
			}
		}
	}
}


- (void) setPrinter: (NSString*) printer {	
	[NSTask launchedTaskWithLaunchPath:@"/usr/bin/lpoptions" arguments:[NSArray arrayWithObjects:@"-d", printer,nil]];
}

- (void) setLocation: (NSString*) location {
	[NSTask launchedTaskWithLaunchPath:@"/usr/sbin/scselect" arguments:[NSArray arrayWithObject:location]];	
}



- (IBAction) showWebpage:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://earthlingsoft.net/hacks.html#locationchecker"]];
}

- (IBAction) showReadme:(id)sender
{
	[[NSWorkspace sharedWorkspace] openFile:[[NSBundle mainBundle] pathForResource:@"Readme" ofType:@"rtf"]];
}

- (IBAction) sendEMail:(id)sender {
	NSString * theURL = [NSString stringWithFormat:@"mailto:earthlingsoft%%40earthlingsoft.net?subject=Location%%20Checker%%20%@", [self myVersionString]];
	NSURL * myURL = [NSURL URLWithString:theURL];
	[[NSWorkspace sharedWorkspace] openURL:myURL];
}

#pragma mark UTILITY

- (NSString*) myVersionString {
	return [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
}



#pragma mark WINDOW DELEGATE

- (void)windowWillClose:(NSNotification *)aNotification {
	[NSApp terminate:self];
}


@end
